@@ -1,6 +1,8 @@
+from .asyncio_endpoint import AsyncioEndpoint
 from .attestation_endpoint import AttestationEndpoint
 from .base_endpoint import BaseEndpoint
 from .dht_endpoint import DHTEndpoint
+from .identity_endpoint import IdentityEndpoint
 from .isolation_endpoint import IsolationEndpoint
 from .network_endpoint import NetworkEndpoint
 from .noblock_dht_endpoint import NoBlockDHTEndpoint
@@ -16,8 +18,10 @@ class RootEndpoint(BaseEndpoint):
     """
 
     def setup_routes(self):
-        endpoints = {'/attestation': AttestationEndpoint,
+        endpoints = {'/asyncio': AsyncioEndpoint,
+                     '/attestation': AttestationEndpoint,
                      '/dht': DHTEndpoint,
+                     '/identity': IdentityEndpoint,
                      '/isolation': IsolationEndpoint,
                      '/network': NetworkEndpoint,
                      '/noblockdht': NoBlockDHTEndpoint,
