@@ -7,27 +7,18 @@ from hashlib import sha1
 from random import choice
 from threading import RLock
 
-from .bonehexact.algorithm import BonehExactAlgorithm
 from .caches import (HashCache, PeerCache, PendingChallengeCache, ProvingAttestationCache,
                      ReceiveAttestationRequestCache, ReceiveAttestationVerifyCache)
 from .database import AttestationsDB
-from .irmaexact.algorithm import IRMAExactAlgorithm
 from .payload import (AttestationChunkPayload, ChallengePayload, ChallengeResponsePayload, RequestAttestationPayload,
                       VerifyAttestationRequestPayload)
-from .pengbaorange.algorithm import PengBaoRangeAlgorithm
-from ..identity_formats import FORMATS
+from ..schema.manager import SchemaManager
 from ...community import Community
 from ...lazy_community import lazy_wrapper
 from ...messaging.payload_headers import BinMemberAuthenticationPayload, GlobalTimeDistributionPayload
 from ...peer import Peer
 from ...requestcache import RequestCache
-from ...util import cast_to_bin, cast_to_chr, cast_to_unicode, maybe_coroutine
-
-ID_ALGORITHMS = {
-    "bonehexact": BonehExactAlgorithm,
-    "irmaexact": IRMAExactAlgorithm,
-    "pengbaorange": PengBaoRangeAlgorithm
-}
+from ...util import cast_to_bin, cast_to_chr, maybe_coroutine
 
 
 def synchronized(f):
@@ -61,6 +52,9 @@ class AttestationCommunity(Community):
 
         self.receive_block_lock = RLock()
 
+        self.schema_manager = SchemaManager()
+        self.schema_manager.register_default_schemas()
+
         self.attestation_request_callback = lambda peer, attribute_name, metadata: None
         self.attestation_request_complete_callback = \
             lambda for_peer, attribute_name, attr_hash, id_format, from_peer=None: None
@@ -95,7 +89,7 @@ class AttestationCommunity(Community):
         self.database.close()
 
     def get_id_algorithm(self, id_format):
-        return ID_ALGORITHMS[FORMATS[id_format]["algorithm"]](id_format)
+        return self.schema_manager.get_algorithm_instance(id_format)
 
     def set_attestation_request_callback(self, f):
         """
@@ -163,7 +157,7 @@ class AttestationCommunity(Community):
             "id_format": id_format
         }
         meta_dict.update(metadata)
-        metadata = cast_to_bin(json.dumps({cast_to_unicode(k): cast_to_unicode(v) for k, v in meta_dict.items()}))
+        metadata = json.dumps(meta_dict).encode()
 
         global_time = self.claim_global_time()
         auth = BinMemberAuthenticationPayload(self.my_peer.public_key.key_to_bin()).to_pack_list()
@@ -250,8 +244,13 @@ class AttestationCommunity(Community):
         """
         We received a request to verify one of our attestations. Send the requested attestation back.
         """
-        attestation_blob, = self.database.get_attestation_by_hash(payload.hash)
+        attestation_blob = self.database.get_attestation_by_hash(payload.hash)
         if not attestation_blob:
+            self.logger.warning("Dropping verification request of unknown hash!")
+            return
+        attestation_blob, = attestation_blob
+        if not attestation_blob:
+            self.logger.warning("Attestation blob for verification is empty!")
             return
 
         value = await maybe_coroutine(self.verify_request_callback, peer, payload.hash)
