@@ -109,7 +109,7 @@ class TunnelSettings(object):
         self.remove_tunnel_delay = 5
 
         self.num_ip_circuits = 3
-        self.peer_flags = PEER_FLAG_RELAY
+        self.peer_flags = {PEER_FLAG_RELAY}
 
 
 class TunnelCommunity(Community):
@@ -160,7 +160,7 @@ class TunnelCommunity(Community):
 
         self.crypto = self.settings.crypto
 
-        self.logger.info("Setting exitnode = %s", self.settings.peer_flags & PEER_FLAG_EXIT_ANY)
+        self.logger.info("Setting exitnode = %s", PEER_FLAG_EXIT_ANY in self.settings.peer_flags)
 
         self.crypto.initialize(self.my_peer.key)
 
@@ -174,11 +174,11 @@ class TunnelCommunity(Community):
     async def unload(self):
         # Remove all circuits/relays/exitsockets
         for circuit_id in list(self.circuits.keys()):
-            self.remove_circuit(circuit_id, 'unload', remove_now=True, destroy=True)
+            self.remove_circuit(circuit_id, 'unload', remove_now=True, destroy=DESTROY_REASON_SHUTDOWN)
         for circuit_id in list(self.relay_from_to.keys()):
-            self.remove_relay(circuit_id, 'unload', remove_now=True, destroy=True, both_sides=False)
+            self.remove_relay(circuit_id, 'unload', remove_now=True, destroy=DESTROY_REASON_SHUTDOWN, both_sides=False)
         for circuit_id in list(self.exit_sockets.keys()):
-            self.remove_exit_socket(circuit_id, 'unload', remove_now=True, destroy=True)
+            self.remove_exit_socket(circuit_id, 'unload', remove_now=True, destroy=DESTROY_REASON_SHUTDOWN)
 
         await self.request_cache.shutdown()
 
@@ -242,7 +242,7 @@ class TunnelCommunity(Community):
                 self.remove_exit_socket(circuit_id, 'traffic limit exceeded')
 
         # Remove candidates that are not returned as verified peers
-        current_peers = set(self.network.get_peers_for_service(self.master_peer.mid))
+        current_peers = self.get_peers()
         for peer in list(self.candidates):
             if peer not in current_peers:
                 self.candidates.pop(peer)
@@ -250,7 +250,7 @@ class TunnelCommunity(Community):
 
     def get_candidates(self, flag):
         return [peer for peer, flags in self.candidates.items()
-                if flags & flag and self.crypto.is_key_compatible(peer.public_key)]
+                if flag in flags and self.crypto.is_key_compatible(peer.public_key)]
 
     def get_max_time(self, circuit_id):
         return self.settings.max_time
@@ -328,7 +328,7 @@ class TunnelCommunity(Community):
         first_hop = random.choice(candidate_list)
         alt_first_hops = [c for c in candidate_list if c != first_hop]
 
-        circuit.unverified_hop = Hop(first_hop.public_key)
+        circuit.unverified_hop = Hop(first_hop.public_key, flags=self.candidates.get(first_hop))
         circuit.unverified_hop.address = first_hop.address
         circuit.unverified_hop.dh_secret, circuit.unverified_hop.dh_first_part = self.crypto.generate_diffie_secret()
 
@@ -357,9 +357,9 @@ class TunnelCommunity(Community):
         self.logger.info("Removing %s circuit %d %s", circuit_to_remove.ctype, circuit_id, additional_info)
 
         if destroy:
-            self.destroy_circuit(circuit_to_remove)
+            self.destroy_circuit(circuit_to_remove, reason=destroy)
 
-        circuit_to_remove.close()
+        circuit_to_remove.close(additional_info)
 
         if not remove_now or self.settings.remove_tunnel_delay > 0:
             await sleep(self.settings.remove_tunnel_delay)
@@ -386,7 +386,7 @@ class TunnelCommunity(Community):
 
         # Send destroy
         if destroy:
-            self.destroy_relay(to_remove, got_destroy_from=got_destroy_from)
+            self.destroy_relay(to_remove, got_destroy_from=got_destroy_from, reason=destroy)
 
         if not remove_now or self.settings.remove_tunnel_delay > 0:
             await sleep(self.settings.remove_tunnel_delay)
@@ -415,7 +415,7 @@ class TunnelCommunity(Community):
         """
         exit_socket_to_destroy = self.exit_sockets.get(circuit_id, None)
         if exit_socket_to_destroy and destroy:
-            self.destroy_exit_socket(exit_socket_to_destroy)
+            self.destroy_exit_socket(exit_socket_to_destroy, reason=destroy)
 
         if not remove_now or self.settings.remove_tunnel_delay > 0:
             await sleep(self.settings.remove_tunnel_delay)
@@ -428,6 +428,7 @@ class TunnelCommunity(Community):
                 await exit_socket.close()
                 # Remove old session key
                 self.relay_session_keys.pop(circuit_id, None)
+            await exit_socket.shutdown_task_manager()
         return exit_socket
 
     def destroy_circuit(self, circuit, reason=0):
@@ -550,8 +551,7 @@ class TunnelCommunity(Community):
         if self.request_cache.has("retry", circuit.circuit_id):
             self.request_cache.pop("retry", circuit.circuit_id)
 
-        ignore_candidates = [self.crypto.key_to_bin(chop.public_key) for chop in circuit.hops] + \
-                            [self.my_peer.public_key]
+        ignore_candidates = [hop.node_public_key for hop in circuit.hops] + [self.my_peer.public_key.key_to_bin()]
         if circuit.required_exit:
             ignore_candidates.append(circuit.required_exit.public_key.key_to_bin())
 
@@ -577,7 +577,8 @@ class TunnelCommunity(Community):
 
         if extend_hop_public_bin:
             extend_hop_public_key = self.crypto.key_from_public_bin(extend_hop_public_bin)
-            circuit.unverified_hop = Hop(extend_hop_public_key)
+            circuit.unverified_hop = Hop(extend_hop_public_key,
+                                         flags=self.candidates.get(Peer(extend_hop_public_bin)))
             circuit.unverified_hop.dh_secret, circuit.unverified_hop.dh_first_part = \
                 self.crypto.generate_diffie_secret()
 
@@ -603,7 +604,7 @@ class TunnelCommunity(Community):
 
     def extract_peer_flags(self, extra_bytes):
         if not extra_bytes:
-            return 0
+            return []
 
         payload = self.serializer.unpack_to_serializables([ExtraIntroductionPayload], extra_bytes)[0]
         return payload.flags
@@ -752,7 +753,7 @@ class TunnelCommunity(Community):
 
     @tc_lazy_wrapper_unsigned(ExtendPayload)
     async def on_extend(self, source_address, payload, _):
-        if not self.settings.peer_flags & PEER_FLAG_RELAY:
+        if PEER_FLAG_RELAY not in self.settings.peer_flags:
             self.logger.warning("Ignoring create for circuit %d", payload.circuit_id)
             return
         if not self.request_cache.has("created", payload.circuit_id):
@@ -775,7 +776,7 @@ class TunnelCommunity(Community):
 
             # Ensure that we are able to contact this peer
             if extend_candidate.last_response + 57.5 < time.time():
-                await self.dht_peer_lookup(extend_candidate.mid)
+                await self.dht_peer_lookup(extend_candidate.mid, peer=extend_candidate)
 
         self.logger.info("On_extend send CREATE for circuit (%s, %d) to %s:%d", source_address,
                          circuit_id, *extend_candidate.address)
@@ -897,15 +898,16 @@ class TunnelCommunity(Community):
         self.logger.info("Got destroy from %s for circuit %s", source_address, circuit_id)
 
         if circuit_id in self.relay_from_to:
-            self.remove_relay(circuit_id, "Got destroy", destroy=True, got_destroy_from=(circuit_id, source_address))
+            self.remove_relay(circuit_id, "got destroy", destroy=DESTROY_REASON_FORWARD,
+                              got_destroy_from=(circuit_id, source_address))
 
         elif circuit_id in self.exit_sockets and source_address == self.exit_sockets[circuit_id].peer.address:
             self.logger.info("Got an exit socket %s %s", circuit_id, source_address)
-            self.remove_exit_socket(circuit_id, "Got destroy")
+            self.remove_exit_socket(circuit_id, f"got destroy with reason {payload.reason}")
 
         elif circuit_id in self.circuits and source_address == self.circuits[circuit_id].peer.address:
             self.logger.info("Got a circuit %s %s", circuit_id, source_address)
-            self.remove_circuit(circuit_id, "Got destroy")
+            self.remove_circuit(circuit_id, f"got destroy with reason {payload.reason}")
 
         else:
             self.logger.warning("Invalid or unauthorized destroy")
@@ -914,12 +916,12 @@ class TunnelCommunity(Community):
         is_ipv8 = DataChecker.could_be_ipv8(data)
         is_ipv8_tunnel = is_ipv8 and self._prefix == data[:22]
 
-        if not (is_ipv8 or self.settings.peer_flags & PEER_FLAG_EXIT_ANY):
+        if not (is_ipv8 or PEER_FLAG_EXIT_ANY in self.settings.peer_flags):
             self.logger.error("Dropping data packets, refusing to be an exit node for data")
 
         elif not (is_ipv8_tunnel
-                  or self.settings.peer_flags & PEER_FLAG_EXIT_IPV8
-                  or self.settings.peer_flags & PEER_FLAG_EXIT_ANY):
+                  or PEER_FLAG_EXIT_IPV8 in self.settings.peer_flags
+                  or PEER_FLAG_EXIT_ANY in self.settings.peer_flags):
             self.logger.error("Dropping data packets, refusing to be an exit node for ipv8")
 
         elif circuit_id in self.exit_sockets:
@@ -943,8 +945,8 @@ class TunnelCommunity(Community):
     def increase_bytes_received(self, obj, num_bytes):
         obj.bytes_down += num_bytes
 
-    async def dht_peer_lookup(self, mid):
+    async def dht_peer_lookup(self, mid, peer=None):
         if self.dht_provider:
-            await self.dht_provider.peer_lookup(mid)
+            await self.dht_provider.peer_lookup(mid, peer)
         else:
             self.logger.error("Need a DHT provider to connect to a peer using the DHT")
