@@ -14,8 +14,9 @@ from .caches import *
 from .community import TunnelCommunity, message_to_payload, tc_lazy_wrapper_unsigned
 from .payload import *
 from .tunnel import (CIRCUIT_ID_PORT, CIRCUIT_TYPE_IP_SEEDER, CIRCUIT_TYPE_RP_DOWNLOADER, CIRCUIT_TYPE_RP_SEEDER,
-                     EXIT_NODE, EXIT_NODE_SALT, Hop, IntroductionPoint, PEER_SOURCE_DHT, PEER_SOURCE_PEX, RelayRoute,
-                     RendezvousPoint, Swarm, TunnelExitSocket)
+                     DESTROY_REASON_LEAVE_SWARM, DESTROY_REASON_UNNEEDED, EXIT_NODE, EXIT_NODE_SALT, Hop,
+                     IntroductionPoint, PEER_SOURCE_DHT, PEER_SOURCE_PEX, RelayRoute, RendezvousPoint, Swarm,
+                     TunnelExitSocket)
 from ...keyvault.public.libnaclkey import LibNaCLPK
 from ...messaging.anonymization.pex import PexCommunity
 from ...messaging.deprecated.encoding import decode, encode
@@ -98,14 +99,14 @@ class HiddenTunnelCommunity(TunnelCommunity):
             # If there are no other swarms with the same hop count, remove the data circuits
             if not [s for s in self.swarms.values() if s != swarm and s.hops == swarm.hops]:
                 for circuit in self.find_circuits(hops=swarm.hops, state=None):
-                    self.remove_circuit(circuit.circuit_id, 'leaving hidden swarm', destroy=True)
+                    self.remove_circuit(circuit.circuit_id, 'not needed', destroy=DESTROY_REASON_UNNEEDED)
             # Remove e2e circuits
             for rp_circuit, _ in swarm.connections.values():
-                self.remove_circuit(rp_circuit.circuit_id, 'leaving hidden swarm', destroy=True)
+                self.remove_circuit(rp_circuit.circuit_id, 'leaving hidden swarm', destroy=DESTROY_REASON_LEAVE_SWARM)
         # Remove introduction points
         for ip_circuit in self.circuits.values():
             if ip_circuit.info_hash == info_hash and ip_circuit.ctype == CIRCUIT_TYPE_IP_SEEDER:
-                self.remove_circuit(ip_circuit.circuit_id, 'leaving hidden swarm', destroy=True)
+                self.remove_circuit(ip_circuit.circuit_id, 'leaving hidden swarm', destroy=DESTROY_REASON_LEAVE_SWARM)
 
     async def estimate_swarm_size(self, info_hash, hops=1, max_requests=10):
         """
@@ -133,7 +134,7 @@ class HiddenTunnelCommunity(TunnelCommunity):
 
             # Issue as many requests as possible in parallel
             ips = random.sample(not_tried, min(len(not_tried), max_requests - len(tried)))
-            responses = await gather(*[swarm.lookup(ip) for ip in ips], return_errors=True)
+            responses = await gather(*[swarm.lookup(ip) for ip in ips], return_exceptions=True)
 
             # Collect responses
             all_ += sum([result for result in responses if not isinstance(result, Exception)], [])
